from sqlalchemy import Column, func, ForeignKey
from sqlalchemy.orm import relationship
from sqlalchemy.dialects.postgresql import UUID, JSON, TIMESTAMP, TEXT, INTEGER, VARCHAR
from uuid import uuid4
from sqlalchemy.ext.declarative import declarative_base


TaskBase = declarative_base()


class Task(TaskBase):
    __tablename__ = "tasks"

    uuid = Column(UUID(as_uuid=True), primary_key=True, default=uuid4)

    identifier = Column(VARCHAR(50), index=True)
    data_schema = Column(JSON)

    max_execution_time = Column(INTEGER)  # Minutes

    created_at = Column(TIMESTAMP, server_default=func.current_timestamp())
    updated_at = Column(TIMESTAMP, onupdate=func.current_timestamp())


TaskResultBase = declarative_base()


class TaskResult(TaskResultBase):
    __tablename__ = "task_results"

    uuid = Column(UUID(as_uuid=True), primary_key=True, default=uuid4)

    task_request_uuid = Column(UUID(as_uuid=True), ForeignKey('task_requests.uuid'), nullable=False,
                               unique=True)

    status = Column(VARCHAR(50), index=True)
    data = Column(JSON)

    created_at = Column(TIMESTAMP, server_default=func.current_timestamp())


TaskStatusBase = declarative_base()


class TaskStatus(TaskStatusBase):
    __tablename__ = "task_statuses"

    uuid = Column(UUID(as_uuid=True), primary_key=True, default=uuid4)

    task_request_uuid = Column(UUID(as_uuid=True), ForeignKey('task_requests.uuid'), nullable=False, index=True)

    status = Column(VARCHAR(50), index=True)
    data = Column(JSON)
    created_at = Column(TIMESTAMP, server_default=func.current_timestamp())


TaskRequestBase = declarative_base()


class TaskRequest(TaskRequestBase):
    __tablename__ = 'task_requests'

    uuid = Column(UUID(as_uuid=True), primary_key=True, default=uuid4)

    task_uuid = Column(UUID(as_uuid=True), ForeignKey(Task.uuid), nullable=False, index=True)
    task = relationship(Task, back_populates="requests")

    result = relationship(TaskResult, uselist=False, back_populates="task_request")

    statuses = relationship(TaskStatus, back_populates="task_request")

    data = Column(JSON)

    callback = Column(TEXT)

    created_at = Column(TIMESTAMP, server_default=func.current_timestamp())
    updated_at = Column(TIMESTAMP, onupdate=func.current_timestamp())
